@@ -5,6 +5,7 @@ import json
 import shutil
 import fnmatch
 import hashlib
+import sys
 import threading
 import concurrent.futures
 import asyncio
@@ -14,12 +15,12 @@ import tiktoken
 import yaml
 from openai import OpenAI
 from playwright.async_api import async_playwright
+import logging
+logger = logging.getLogger('robodog.service')
 
 class RobodogService:
     def __init__(self, config_path: str, api_key: str = None):
         self._load_config(config_path)
-        self.context   = ""
-        self.knowledge = ""
         self.stashes   = {}
         self._init_llm(api_key)
 
@@ -55,11 +56,9 @@ class RobodogService:
 
     # ————————————————————————————————————————————————————————————
     # CORE LLM / CHAT
-    def ask(self, prompt: str) -> str:
+    def ask2(self, prompt: str) -> str:
+        logger.debug(f"ask {prompt!r}")
         messages = [
-            {"role":"system","content":"You are Robodog, a helpful assistant."},
-            {"role":"system","content":"Chat History:\n"+self.context},
-            {"role":"system","content":"Knowledge Base:\n"+self.knowledge},
             {"role":"user","content":prompt},
         ]
         resp = self.client.chat.completions.create(
@@ -83,6 +82,41 @@ class RobodogService:
             answer = resp.choices[0].message.content.strip()
         return answer
 
+    # ————————————————————————————————————————————————————————————
+    # CORE LLM / CHAT
+    def ask(self, prompt: str) -> str:
+        logger.debug(f"ask {prompt!r}")
+        messages = [{"role":"user","content":prompt}]
+        resp = self.client.chat.completions.create(
+            model=self.cur_model,
+            messages=messages,
+            temperature=self.temperature,
+            top_p=self.top_p,
+            frequency_penalty=self.frequency_penalty,
+            presence_penalty=self.presence_penalty,
+            stream=self.stream,
+        )
+        # spinner sequence of thinking animals
+        spinner = ["🐶", "🐱", "🦊", "🐵", "🦄"]
+        idx = 0
+        answer = ""
+        if self.stream:
+            for chunk in resp:
+                delta = getattr(chunk.choices[0].delta, "content", None)
+                if delta:
+                    answer += delta
+                # display latest line + spinner
+                last_line = answer.splitlines()[-1] if "\n" in answer else answer
+                sys.stdout.write(f"\r{spinner[idx % len(spinner)]} {last_line[:60]}{'…' if len(last_line)>60 else ''}")
+                sys.stdout.flush()
+                idx += 1
+            # final newline
+            sys.stdout.write("\n")
+        else:
+            answer = resp.choices[0].message.content.strip()
+        return answer
+
+
     # ————————————————————————————————————————————————————————————
     # MODEL / KEY MANAGEMENT
     def list_models(self):
@@ -105,30 +139,30 @@ class RobodogService:
     # ————————————————————————————————————————————————————————————
     # STASH / POP / LIST / CLEAR / IMPORT / EXPORT
     def stash(self, name: str):
-        self.stashes[name] = (self.context, self.knowledge)
+        self.stashes[name] = str
 
     def pop(self, name: str):
         if name not in self.stashes:
             raise KeyError(f"No stash {name}")
-        self.context, self.knowledge = self.stashes[name]
+        return self.stashes[name]
 
     def list_stashes(self):
         return list(self.stashes.keys())
 
     def clear(self):
-        self.context = ""
-        self.knowledge = ""
+        ""
 
     def import_files(self, glob_pattern: str) -> int:
         count = 0
+        knowledge = ""
         for fn in __import__('glob').glob(glob_pattern, recursive=True):
             try:
                 txt = open(fn, 'r', encoding='utf-8', errors='ignore').read()
-                self.knowledge += f"\n\n--- {fn} ---\n{txt}"
+                knowledge += f"\n\n--- {fn} ---\n{txt}"
                 count += 1
             except:
                 pass
-        return count
+        return knowledge
 
     def export_snapshot(self, filename: str):
         with open(filename, 'w', encoding='utf-8') as f:
@@ -187,6 +221,7 @@ class RobodogService:
 
     def include(self, spec_text: str, prompt: str = None):
         inc = self.parse_include(spec_text)
+        knowledge = ""
         # build search payloads
         searches = []
         if inc["type"] == "dir":
@@ -208,9 +243,10 @@ class RobodogService:
         included_txts = []
         def _read(path):
             blob = self.call_mcp("READ_FILE", {"path": path})
-            return blob.get("content","")
+            return path, blob.get("content", "")
+
         with concurrent.futures.ThreadPoolExecutor(max_workers=8) as pool:
-            for txt in pool.map(_read, matches):
+            for path, txt in pool.map(_read, matches):
                 # token count
                 try:
                     enc = tiktoken.encoding_for_model(self.cur_model)
@@ -218,20 +254,151 @@ class RobodogService:
                     enc = tiktoken.get_encoding("gpt2")
                 wc = len(txt.split())
                 tc = len(enc.encode(txt))
-                print(f"Included: words={wc}, tokens={tc}")
+                # fixed print statement
+                print(f"Included: {path} ({wc} tokens)")
                 included_txts.append(txt)
-        combined = "\n".join(included_txts)
-        self.knowledge += "\n" + combined + "\n"
+                combined = "\n".join(included_txts)
+                knowledge += "\n" + combined + "\n"
 
-        # if prompt, ask & update context
-        if prompt:
-            print(f"→ Prompt: {prompt}")
-            self.context += f"\nUser: {prompt}"
-            ans = self.ask(prompt)
-            self.context += f"\nAI: {ans}"
-            return ans
-        return None
+        return knowledge
+    
+    # ----------------------------------------------------------------
+    DEFAULT_EXCLUDE_DIRS = {"node_modules", "dist"}
+    logger = logging.getLogger(__name__)
 
+    def search_files(self,
+                    patterns="*",
+                    recursive=True,
+                    roots=None,
+                    exclude_dirs=None):
+        """
+        patterns: a string (e.g. '*.js|*.jsx') or a list of patterns
+        recursive: whether to walk subdirectories
+        roots:       list of root folders to search
+        exclude_dirs: iterable of dir‐names to skip (default: node_modules, dist)
+        """
+        # Normalize inputs
+        self.logger.debug("Raw patterns input: %r", patterns)
+        if isinstance(patterns, str):
+            patterns = patterns.split("|")
+        else:
+            patterns = list(patterns)
+        self.logger.debug("Normalized patterns list: %r", patterns)
+
+        exclude_dirs = set(exclude_dirs or self.DEFAULT_EXCLUDE_DIRS)
+        self.logger.debug("Excluding directories: %r", exclude_dirs)
+
+        matches = []
+
+        for root in roots or []:
+            self.logger.debug(">> Entering root: %r", root)
+            if not os.path.isdir(root):
+                self.logger.debug("   Skipping %r because it is not a directory", root)
+                continue
+
+            if recursive:
+                for dirpath, dirnames, filenames in os.walk(root):
+                    self.logger.debug("Walking into %r, subdirs=%r, files=%r",
+                                dirpath, dirnames, filenames)
+
+                    # prune excluded dirs in-place
+                    before = list(dirnames)
+                    dirnames[:] = [d for d in dirnames if d not in exclude_dirs]
+                    removed = set(before) - set(dirnames)
+                    if removed:
+                        self.logger.debug("   Pruned dirs %r from %r", removed, dirpath)
+
+                    for fn in filenames:
+                        full_path = os.path.join(dirpath, fn)
+                        self.logger.debug("   Considering file: %r", full_path)
+                        matched = False
+                        for pat in patterns:
+                            if fnmatch.fnmatch(full_path, pat) or fnmatch.fnmatch(fn, pat):
+                                self.logger.debug("      MATCH: %r matches pattern %r", full_path, pat)
+                                matches.append(full_path)
+                                matched = True
+                                break
+                            else:
+                                self.logger.debug("      no match: %r !~ %r", full_path, pat)
+                        if not matched:
+                            self.logger.debug("   -> File %r did not match any pattern", full_path)
+
+            else:
+                self.logger.debug("Non-recursive mode in root: %r", root)
+                for fn in os.listdir(root):
+                    full = os.path.join(root, fn)
+                    self.logger.debug("   Top-level entry: %r", full)
+                    if not os.path.isfile(full):
+                        self.logger.debug("      Skipped %r (not a file)", full)
+                        continue
+                    if fn in exclude_dirs:
+                        self.logger.debug("      Skipped %r (in exclude_dirs)", fn)
+                        continue
+
+                    matched = False
+                    for pat in patterns:
+                        if fnmatch.fnmatch(full, pat) or fnmatch.fnmatch(fn, pat):
+                            self.logger.debug("      MATCH: %r matches pattern %r", full, pat)
+                            matches.append(full)
+                            matched = True
+                            break
+                        else:
+                            self.logger.debug("      no match: %r !~ %r", full, pat)
+                    if not matched:
+                        self.logger.debug("   -> File %r did not match any pattern", full)
+
+        self.logger.debug("Search complete. Found %d matches: %r", len(matches), matches)
+        return matches
+    def search_files3(self,
+                     patterns="*",
+                     recursive=True,
+                     roots=None,
+                     exclude_dirs=None):
+        """
+        patterns: a string (e.g. '*.js|*.jsx') or a list of patterns
+        recursive: whether to walk subdirectories
+        roots:       list of root folders to search
+        exclude_dirs: iterable of dir‐names to skip (default: node_modules, dist)
+        """
+        # normalize inputs
+        if isinstance(patterns, str):
+            patterns = patterns.split("|")
+        else:
+            patterns = list(patterns)
+
+        exclude_dirs = set(exclude_dirs or self.DEFAULT_EXCLUDE_DIRS)
+        matches = []
+
+        for root in roots or []:
+
+            if not os.path.isdir(root):
+                continue
+
+            if recursive:
+                for dirpath, dirnames, filenames in os.walk(root):
+                    # prune excluded dirs in-place
+                    dirnames[:] = [d for d in dirnames if d not in exclude_dirs]
+
+                    for fn in filenames:
+                        # quick basename match
+                        for pat in patterns:
+                            if fnmatch.fnmatch(fn, pat):
+                                matches.append(os.path.join(dirpath, fn))
+                                break
+            else:
+                # non-recursive: only top-level files
+                for fn in os.listdir(root):
+                    full = os.path.join(root, fn)
+                    if not os.path.isfile(full):
+                        continue
+                    if fn in exclude_dirs:
+                        continue
+                    for pat in patterns:
+                        if fnmatch.fnmatch(fn, pat):
+                            matches.append(full)
+                            break
+
+        return matches
     # ————————————————————————————————————————————————————————————
     # /CURL IMPLEMENTATION
     def curl(self, tokens: list):
